#include "crogue.h"

const uchar save_file_extension[6] = { 0,'S','A','V',0,0xf8 };

//{{{
//
// Save the game state
// Presently, just byte-dumps the main data structure
// This needs to be changed to include compression for the
// bigger parts.
//
void save_game(void)
{
	FILE *sg;
	
	// If a previous version of this game exists, delete it before writing new one
#ifdef IS_CALCULATOR
	SymDel(SAVEFILENAME_TIOS);
#endif
	
	sg = fopen(SAVEFILENAME, "wb");
	if(sg)
	{
		fwrite(w, sizeof(world), 1, sg);
		fwrite(w->shuffletranslation, sizeof(uint), w->desc.shuffletabsize, sg);
		fwrite(w->itemids, 32, 1, sg);
		fwrite(w->items.items, sizeof(item), w->items.alloced, sg);
		fwrite(w->plr.spellknowledge, sizeof(uint), w->desc.numspells, sg);
#		ifdef IS_CALCULATOR
		fwrite(save_file_extension, 6, 1, sg);
#		endif
		fclose(sg);
#ifdef IS_CALCULATOR
		EM_moveSymToExtMem(SAVEFILENAME_TIOS, HS_NULL);
		FontSetSys(F_4x6);
#endif
	}
}
//}}}
//{{{
//
// Attempts to load a saved game from the filesystem.
// Returns 1 if successful or 0 if no saved game exists.
//
int restore_game(void)
{
	FILE *fin;
	
#ifdef IS_CALCULATOR
	EM_moveSymFromExtMem(SAVEFILENAME_TIOS, HS_NULL);
#endif
	fin = fopen(SAVEFILENAME, "rb");
	if( fin )
	{
		fread(w, sizeof(world), 1, fin);
		
		w->shuffletranslation = (uint*)malloc( sizeof(uint) * w->desc.shuffletabsize );
		if(!w->shuffletranslation)
			fatal("Out of memory.");
		fread(w->shuffletranslation, sizeof(uint), w->desc.shuffletabsize, fin);
		
		w->itemids = (ulong*)malloc( 32 );
		if(!w->itemids)
			fatal("Out of memory.");
		fread(w->itemids, 32, 1, fin);
		
		w->items.items = (item*)malloc(w->items.alloced*sizeof(item));
		if(!w->items.items)
			fatal("Out of memory.");
		fread(w->items.items, sizeof(item), w->items.alloced, fin);
		
		init_spells();
		fread(w->plr.spellknowledge, sizeof(uint), w->desc.numspells, fin);
		
		fclose(fin);
		return 1;
	}
	return 0;
}
//}}}

const uchar level_file_extension[6] = { 0,'L','E','V',0,0xf8 };

//{{{
//
// Save the current level (the parts of the game state which should be restored
// when the player comes back: items, monsters, walls/floor, etc). The two large
// tables (monsters and tiles) are written using a special form of run-length
// encoding. The first byts of each struct are ordered sequentially, followed
// by the second byte of each struct, etc. This avoids the complexity of having
// a single run length handle more than one character, while keeping similar
// bytes together.
//
// Files are named by the convention ROGLVL## where ## is the level's number.
//
void save_level(void)
{
	FILE *save;
	
#ifdef IS_CALCULATOR
	char name[10]="\0ROGLVL  ";
	name[7] = (w->level / 10) + '0';
	name[8] = (w->level % 10) + '0';
	
	// Prep for efficient writing
	compress_items();
	compress_monsters();
	
	// Unlink pre-existing level of the same name
	SymDel(name+9);
	
	save = fopen(name+1, "wb");
#else
	char name[13]="roglvl  .sav";
	name[6] = (w->level / 10) + '0';
	name[7] = (w->level % 10) + '0';
	
	save = fopen(name, "wb");
#endif	

	frleputc( 0, save, 2 ); 
	
	frlewrite( (char*)(&w->m[0]), sizeof(monster), MONSTERS_MAX, save );
	frlewrite( (char*)(&w->t[0][0]), sizeof(tile), MAPSIZE_X * MAPSIZE_Y, save );
	
	// Write items
	frlewrite(&w->items, sizeof(itemsinfo), 1, save);
	frlewrite(w->items.items, sizeof(item), w->items.alloced, save);
	
	frleputc( 0, save, 1 );
	
#ifdef IS_CALCULATOR
	fwrite(level_file_extension, 6, 1, save);
#endif
	fclose(save);

#ifdef IS_CALCULATOR	
	EM_moveSymToExtMem( name+9, HS_NULL );
	FontSetSys(F_4x6);
#endif
}
//}}}
//{{{
//
// Load the level number given in w->level from a run-length encoded file.
// Returns 1 if successful, 0 if the file does not exist.
//
int load_level(int source, coord pos)
{
	FILE *save;
	
#ifdef IS_CALCULATOR
	char name[10]="ROGLVL  ";
#else
	char name[13]="roglvl  .sav";
#endif
	name[6] = (w->level / 10) + '0';
	name[7] = (w->level % 10) + '0';
	
	save = fopen(name, "rb");
	
	if( !save )
		return 0;
	
	frleread( (char*)(&w->m[0]), sizeof(monster), MONSTERS_MAX, save );
	frleread( (char*)(&w->t[0][0]), sizeof(tile), MAPSIZE_X * MAPSIZE_Y, save );
	
	place_player_on_map(source, pos);
	
	// Read items
	cleanup_items();
	frleread(&w->items, sizeof(itemsinfo), 1, save);
	w->items.items = (item*)malloc(w->items.alloced*sizeof(item));
	frleread(w->items.items, sizeof(item), w->items.alloced, save);
	
	fclose(save);
	
	UI_TF_FullRedraw(&w->tilefield);
	
	return 1;
}
//}}}
//{{{
//
// Delete any remnants of saved games (the saved game and saved levels). This
// is called when the player dies to prevent him from using the files to cheat
// and bring his character back.
//
void cleanup_save(void)
{
#ifdef IS_CALCULATOR
	int i;
	char name[10]="\0ROGLVL  ";
	
	EM_moveSymFromExtMem(SAVEFILENAME_TIOS, HS_NULL);
	SymDel(SAVEFILENAME_TIOS);
	
	for(i=1; i<=w->maxlevel; i++)
	{
		name[7] = (i / 10) + '0';
		name[8] = (i % 10) + '0';
		EM_moveSymFromExtMem(name+9, HS_NULL);
		SymDel(name+9);
	}
#else
	int i;
	char name[13]="roglvl  .sav";
	
	remove(SAVEFILENAME);
	
	for(i=1; i<=w->maxlevel+1; i++)
	{
		name[6] = (i / 10) + '0';
		name[7] = (i % 10) + '0';
		remove(name);
	}
#endif
}
//}}}

